import { RelativeMiddlewareOptions, SerializeMiddleware } from "@smithy/types";
import { PreviouslyResolved } from "./region-redirect-middleware";
export declare const regionRedirectEndpointMiddleware: (
  config: PreviouslyResolved
) => SerializeMiddleware<any, any>;
export declare const regionRedirectEndpointMiddlewareOptions: RelativeMiddlewareOptions;
