import { RelativeMiddlewareOptions, SerializeMiddleware } from "@smithy/types";
interface PreviouslyResolved {
  bucketEndpoint?: boolean;
}
export declare function bucketEndpointMiddleware(
  options: PreviouslyResolved
): SerializeMiddleware<any, any>;
export declare const bucketEndpointMiddlewareOptions: RelativeMiddlewareOptions;
export {};
