import {
  FinalizeRequestHandlerOptions,
  FinalizeRequestMiddleware,
  Pluggable,
} from "@smithy/types";
export declare function checkContentLengthHeader(): FinalizeRequestMiddleware<
  any,
  any
>;
export declare const checkContentLengthHeaderMiddlewareOptions: FinalizeRequestHandlerOptions;
export declare const getCheckContentLengthHeaderPlugin: (
  unused: any
) => Pluggable<any, any>;
